package externaloutage.view;


import com.oracle.ugbu.ss.SystemProperties;
import com.oracle.ugbu.ss.outage.geomap.view.backing.OutageManagedBean;
import com.oracle.ugbu.ss.shared.view.helper.ADFDCHelper;

import java.util.Map;

import javax.faces.application.ViewHandler;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.event.ValueChangeEvent;

import oracle.adf.model.bean.DCDataRow;
import oracle.adf.model.binding.DCIteratorBinding;
import oracle.adf.share.logging.ADFLogger;

import oracle.binding.OperationBinding;

import oracle.jbo.Row;


public class ExternalOutageManagedBean extends OutageManagedBean {

    private static final ADFLogger _LOG =
        ADFLogger.createADFLogger(ExternalOutageManagedBean.class);
    private static final String CLASS_NAME =
        ExternalOutageManagedBean.CLASS_NAME;
    @SuppressWarnings("compatibility:5775185594481036510")
    private static final long serialVersionUID = 1L;
    private ADFDCHelper mDCHelper = ADFDCHelper.getInstance();
    private String outageString;
    private String outageInfoString;
    private String mDefaultAreaType;


    public ExternalOutageManagedBean() {
        super();
    }

    public void setOutageString(String outageString) {
        this.outageString = outageString;
    }

    public String getOutageString() {
        final String METHOD_NAME = "getOutageString";
        _LOG.entering(CLASS_NAME, METHOD_NAME);

        DCIteratorBinding outageAreas =
            mDCHelper.getItrtBindings("GeographicOutageAreasListIterator");

        Row[] allRowsInRange = outageAreas.getAllRowsInRange();
        int i = 0;
        outageString = "";
        if (allRowsInRange != null) {
            ExternalOutageSummaryInfoBean summaryOutage;
            for (Row row : allRowsInRange) {
                DCDataRow dcRow = (DCDataRow)row;
                summaryOutage =
                        new ExternalOutageSummaryInfoBean((Map<String, String>)dcRow.getDataProvider());
                if (i > 0)
                    outageString = outageString.concat("|");
                outageString =
                        outageString.concat(summaryOutage.getOutageArea());
                System.out.println("OUTAGE AREA IS : " +
                                   summaryOutage.getOutageArea());
                i++;
            }
        }
        _LOG.exiting(CLASS_NAME, METHOD_NAME, outageString);
        return outageString;
    }

    public String getOutageInfoString() {
        final String METHOD_NAME = "getOutageInfoString";
        _LOG.entering(CLASS_NAME, METHOD_NAME);

        DCIteratorBinding outageAreas =
            mDCHelper.getItrtBindings("GeographicOutageAreasListIterator");

        Row[] allRowsInRange = outageAreas.getAllRowsInRange();
        int i = 0;
        outageInfoString = "";
        if (allRowsInRange != null) {
            ExternalOutageSummaryInfoBean summaryOutage;
            for (Row row : allRowsInRange) {
                DCDataRow dcRow = (DCDataRow)row;
                summaryOutage =
                        new ExternalOutageSummaryInfoBean((Map<String, String>)dcRow.getDataProvider());
                if (i > 0)
                    outageInfoString = outageInfoString.concat("|");
                outageInfoString =
                        outageInfoString.concat(getInfoHTML(summaryOutage));
                System.out.println("OUTAGE INFO IS : " + outageInfoString);
                i++;
            }
        }
        _LOG.exiting(CLASS_NAME, METHOD_NAME, outageString);
        return outageInfoString;
    }

    public String getDefaultAreaType() {
        final String METHOD_NAME = "getDefaultAreaType";
        _LOG.entering(CLASS_NAME, METHOD_NAME);

        if (mDefaultAreaType == null) {
            mDefaultAreaType =
                    SystemProperties.getSSProperty("outage.area.nms.config");
            _LOG.finest("Setting default Area Type ", CLASS_NAME,
                        mDefaultAreaType);
        }

        _LOG.exiting(CLASS_NAME, METHOD_NAME);
        return mDefaultAreaType;
    }

    protected void refreshPage() {
        FacesContext fc = FacesContext.getCurrentInstance();
        String refreshpage = fc.getViewRoot().getViewId();
        ViewHandler ViewH = fc.getApplication().getViewHandler();
        UIViewRoot UIV = ViewH.createView(fc, refreshpage);
        UIV.setViewId(refreshpage);
        fc.setViewRoot(UIV);
    }

    public void mobileOutageByChanged(ValueChangeEvent valueChangeEvent) {
        setDefaultAreaType(valueChangeEvent.getNewValue().toString());
        ADFDCHelper DCHelper = ADFDCHelper.getInstance();
        DCIteratorBinding payloadIterator =
            mDCHelper.getItrtBindings("payloadIterator");
        Row payloadRow = payloadIterator.getCurrentRow();
        Map payloadDataProvider =
            (Map)((DCDataRow)payloadRow).getDataProvider();
        payloadDataProvider.put("areaType",
                                valueChangeEvent.getNewValue().toString());

        OperationBinding operBinding =
            DCHelper.getBindings().getOperationBinding("process");
        Object operBindingResult = operBinding.execute();
        if (operBinding.getErrors().size() > 0) {
            System.out.println("Error in Outage invoke");
        } else {
            System.out.println("Outage invoked successfully");
        }

        refreshPage();
    }

    private String getInfoHTML(ExternalOutageSummaryInfoBean outagesummaryinfobean) {
        String htmlString =
            "<table border='1' class=\"infotable\"><tr><td>Area</td><td>" +
            outagesummaryinfobean.getOutageArea() +
            "</td></tr><tr><td>Number of Outages</td><td>" +
            outagesummaryinfobean.getNumOutages() +
            "</td></tr><tr><td>Number of Customers Affected</td><td>" +
            outagesummaryinfobean.getNumCustomersAffected() +
            "</td></tr><tr><td>Estimated Fix Time</td><td>" +
            outagesummaryinfobean.getEstimatedFixTime() + "</td></tr></table>";
        return htmlString;
    }


    public void setOutageInfoString(String outageInfoString) {
        this.outageInfoString = outageInfoString;
    }

    public void setDefaultAreaType(String mDefaultAreaType) {
        this.mDefaultAreaType = mDefaultAreaType;
    }

}
